clc; format short g; clear all; close all;

% import data from Excel
X = importdata('dataset.xlsx')


% define variables for the VAR
%--------------------------------------------------------------------------
t0 = 1;                                                                     % start date: 1960:Q1 
T = 235;                                                                    % end data: (196=2007:Q4) (235=2017:Q3)

% general macro variables
y    = log(X.data.Macro(t0:T,1))*100;                                       % y    = log of real GDP
p    = log(X.data.Macro(t0:T,2)/100)*100;                                   % p    = log of chain type GDP Price Index

i    = (log(X.data.Real(1,t0:T))*100)';                                     % i = log of real private fixed investment
q    = (log(X.data.Price(1,t0:T)/100)*100)';                                % q = log of price index for private fixed investment

ff   = X.data.Macro(t0:T,3);                                                % ff   = shadow federal funds rate (Wu-Xia)    
m1   = log(X.data.Macro(t0:T,6))*100;                                       % m1   = log of M1 money stock

ppi_fuel = log(X.data.Macro(t0:T,4)/100)*100;                               % ppi_mat = log of PPI (fuels and related products) NSA
ppi_fin = log(X.data.Macro(t0:T,5)/100)*100;                                % ppi_fin = log of PPI (final demand for finished goods) SA
%--------------------------------------------------------------------------


% create a VAR(p) specification and estimate
%--------------------------------------------------------------------------
Y = [y p i q ppi_fuel-ppi_fin ff m1];

Spec = vgxset('n',size(Y,2),'nAR',4,'Constant',true)
n = Spec.n; plag = Spec.nAR;

[EstSpec,EstStdErrors,LLF,W] = vgxvarx(Spec,Y(plag+1:end,:),[],Y(1:plag,:));        % Warning! By default vgxvarx.m pads presample observations with zeros.      
EstSpec
vgxdisp(EstSpec)
vgxdisp(EstStdErrors);
AIC = LLF - (n + plag*(n^2))
BIC = LLF - ((n + plag*(n^2))/2)*log(size(Y,1)-plag)
%--------------------------------------------------------------------------


% apply recursive identification of monetary policy shocks
%--------------------------------------------------------------------------
StructSpec = recursiveness(EstSpec)

DynRespFunc = impulse(EstSpec,StructSpec,6,20)
%--------------------------------------------------------------------------


% Compute confidence intervals for impulse response functions
%--------------------------------------------------------------------------
% method: simulate from a guassian distribution
tic
errorbands = irfbands_guassian(Y,EstSpec,DynRespFunc,0.10,10000,2000,0)
toc
%--------------------------------------------------------------------------


% record dynamic response functions
%--------------------------------------------------------------------------
irf_y   = -DynRespFunc.Phi{1};   irf_y_LB   = -errorbands.UB{1}; irf_y_UB   = -errorbands.LB{1};
irf_p   = -DynRespFunc.Phi{2};   irf_p_LB   = -errorbands.UB{2}; irf_p_UB   = -errorbands.LB{2};
irf_i   = -DynRespFunc.Phi{3};   irf_i_LB   = -errorbands.UB{3}; irf_i_UB   = -errorbands.LB{3};
irf_q   = -DynRespFunc.Phi{4};   irf_q_LB   = -errorbands.UB{4}; irf_q_UB   = -errorbands.LB{4};
irf_pf  = -DynRespFunc.Phi{5};   irf_pf_LB  = -errorbands.UB{5}; irf_pf_UB  = -errorbands.LB{5};
irf_ff  = -DynRespFunc.Phi{6};   irf_ff_LB  = -errorbands.UB{6}; irf_ff_UB  = -errorbands.LB{6};
irf_m1  = -DynRespFunc.Phi{7};   irf_m1_LB  = -errorbands.UB{7}; irf_m1_UB  = -errorbands.LB{7};
%--------------------------------------------------------------------------


% plot Fig. 2
%--------------------------------------------------------------------------
irf_y   = irf_y(1:17);     irf_y_LB   = irf_y_LB(1:17);     irf_y_UB   = irf_y_UB(1:17);
irf_p   = irf_p(1:17);     irf_p_LB   = irf_p_LB(1:17);     irf_p_UB   = irf_p_UB(1:17);
irf_i   = irf_i(1:17);     irf_i_LB   = irf_i_LB(1:17);     irf_i_UB   = irf_i_UB(1:17);
irf_q   = irf_q(1:17);     irf_q_LB   = irf_q_LB(1:17);     irf_q_UB   = irf_q_UB(1:17);
irf_pf  = irf_pf(1:17);    irf_pf_LB  = irf_pf_LB(1:17);    irf_pf_UB  = irf_pf_UB(1:17);
irf_ff  = irf_ff(1:17);    irf_ff_LB  = irf_ff_LB(1:17);    irf_ff_UB  = irf_ff_UB(1:17);
irf_m1  = irf_m1(1:17);    irf_m1_LB  = irf_m1_LB(1:17);    irf_m1_UB  = irf_m1_UB(1:17);

periods = 0:1:16; zero = zeros(1,17);

figure

subplot(2,2,1)
plot(periods,irf_y,'k-',periods,zero,'k-'), title('\bf{GDP}','FontSize',8), 
jbfill(periods,irf_y_UB,irf_y_LB,'k','k',1,0.25)
xlabel('quarters after shock','FontSize',8), ylabel('Percent','FontSize',8),
set(gca,'YAxisLocation','right','FontSize',8), xlim([0 16.5]), ylim([-0.5 1]),
box off  

subplot(2,2,2)
plot(periods,irf_p,'k-',periods,zero,'k-'), title('\bf{P}','FontSize',8), 
jbfill(periods,irf_p_UB,irf_p_LB,'k','k',1,0.25)
xlabel('quarters after shock','FontSize',8), ylabel('Percent','FontSize',8),
set(gca,'YAxisLocation','right','FontSize',8), xlim([0 16.5]), ylim([-0.5 1]),
box off  

subplot(2,2,3)
plot(periods,irf_ff,'k-',periods,zero,'k-'), title('\bf{FFR}','FontSize',8), 
jbfill(periods,irf_ff_UB,irf_ff_LB,'k','k',1,0.25)
xlabel('quarters after shock','FontSize',8), ylabel('Percentage Points','FontSize',8),
set(gca,'YAxisLocation','right','FontSize',8), xlim([0 16.5]), ylim([-1 1]),
box off 

subplot(2,2,4)
plot(periods,irf_m1,'k-',periods,zero,'k-'), title('\bf{M1}','FontSize',8), 
jbfill(periods,irf_m1_UB,irf_m1_LB,'k','k',1,0.25)
xlabel('quarters after shock','FontSize',8), ylabel('Percent','FontSize',8),
set(gca,'YAxisLocation','right','FontSize',8), xlim([0 16.5]), ylim([-1 1.5]),
box off 
%--------------------------------------------------------------------------



