clc; format short g; clear all; close all;


% import data from Excel
X = importdata('responsefunctions.xlsx')


% determine significance across all 64 PFI categories
%--------------------------------------------------------------------------
T = 21;

i_index = [6 9 10 11 12 13 14 15 16 19 20 21 23 24 26 27 28 29 31 32 33 34 35 ...
           39 40 42 43 44 45 46 48 50 51 52 53 54 55 57 60 61 62 63 66 67 69 70 72 73 74 75 77 78 79 80 ...
           82 83 84 ...
           88 89 91 92 93 94 ...
           96];

irf_II_LB = X.data.QbandsLB(i_index,:); 
irf_II_UB = X.data.QbandsUB(i_index,:); 
irf_QQ_LB = X.data.PbandsLB(i_index,:);
irf_QQ_UB = X.data.PbandsUB(i_index,:);    

N    = length(i_index);
Isig = zeros(N,T);
Qsig = Isig;

for j = 1:1:N;
    
    temp1   = irf_II_LB(j,:).*irf_II_UB(j,:);
    temp2   = irf_QQ_LB(j,:).*irf_QQ_UB(j,:);
    
    for s = 1:1:T;
        if temp1(s) > 0;
            Isig(j,s) = 1;
        end;
        if temp2(s) > 0;
            Qsig(j,s) = 1;
        end;
    end;
    
end;
%--------------------------------------------------------------------------


% obtain standard deviations (estimated and simulated)
%--------------------------------------------------------------------------
SD_irfII = std(X.data.Quantity(i_index,1:end-3));
SD_irfQQ = std(X.data.Price(i_index,1:end-3));

load II
load QQ
%--------------------------------------------------------------------------


% plot Fig. 5
%--------------------------------------------------------------------------
periods = 0:1:size(Isig,2)-1;

figure,

subplot(1,2,1)
plot(periods,sum(Isig,1)/N,'k-',periods,sum(Qsig,1)/N,'k--'),
title('Fraction of Significant PFI Price and Quantity Responses','FontSize',8)
xlabel('quarters after shock','FontSize',8), xlim([0 16]), ylim([0 1])

subplot(1,2,2)
plot(periods,SD_irfII,'k-',periods,II,'k-',periods,SD_irfQQ,'k--',periods,QQ,'k--')
title('Estimated vs. Simulated Standard Deviations','FontSize',8)
xlabel('quarters after shock','FontSize',8), xlim([0 16]), ylim([0 2])
%--------------------------------------------------------------------------


