clc; format short g; clear all; close all;


% import data from Excel
X = importdata('dataset.xlsx')


% define variables for the VAR
%--------------------------------------------------------------------------
t0 = 1;                                                                     % start date: 1960:Q1 
T = 235;                                                                    % end data: (196=2007:Q4) (235=2017:Q3)

% general macro variables
y    = log(X.data.Macro(t0:T,1))*100;                                       % y    = log of real GDP
p    = log(X.data.Macro(t0:T,2)/100)*100;                                   % p    = log of chain type GDP Price Index

i    = (log(X.data.Real(1,t0:T))*100)';                                     % i = log of real private fixed investment
q    = (log(X.data.Price(1,t0:T)/100)*100)';                                % q = log of price index for private fixed investment

ff   = X.data.Macro(t0:T,3);                                                % ff   = shadow federal funds rate (Wu-Xia)    
m1   = log(X.data.Macro(t0:T,6))*100;                                       % m1   = log of M1 money stock

ppi_fuel = log(X.data.Macro(t0:T,4)/100)*100;                               % ppi_mat = log of PPI (fuels and related products) NSA
ppi_fin  = log(X.data.Macro(t0:T,5)/100)*100;                               % ppi_fin = log of PPI (final demand for finished goods) SA
%--------------------------------------------------------------------------


% create a VAR(p) specification and estimate
%--------------------------------------------------------------------------
Y = [y p i q ppi_fuel-ppi_fin ff m1];

% VAR(4) with constant intercept 
%-------------------------------
Spec = vgxset('n',size(Y,2),'nAR',4,'Constant',true)
n = Spec.n; plag = Spec.nAR;

[EstSpec,EstStdErrors,LLF,W] = vgxvarx(Spec,Y(plag+1:end,:),[],Y(1:plag,:));        % Warning! By default vgxvarx.m pads presample observations with zeros.      
EstSpec
%-------------------------------
%--------------------------------------------------------------------------


index = [3:1:34 36:1:56 59:1:83 85:1:93 95 99 100];
N     = length(index);
VD_II = zeros(100,16);
VD_QQ = VD_II;

for j = 1:1:N;
    
% estimate industry-level regressions
%--------------------------------------------------------------------------    
Quantity = (log(X.data.Real(index(j),t0:T))*100)';
Price    = (log(X.data.Price(index(j),t0:T))*100)';

Q1 = lag(Quantity,1); Q2 = lag(Quantity,2); Q3 = lag(Quantity,3); Q4 = lag(Quantity,4); 
P1 = lag(Price,1);    P2 = lag(Price,2);    P3 = lag(Price,3);    P4 = lag(Price,4);
Y1 = lag(Y,1);        Y2 = lag(Y,2);        Y3 = lag(Y,3);        Y4 = lag(Y,4);

YQP = [Y1 Q1 P1 Y2 Q2 P2 Y3 Q3 P3 Y4 Q4 P4]; 
YQP = YQP(plag+1:end,:);

stats1 = regstats(Quantity(plag+1:end),YQP);  
CoeffTable1 = dataset({stats1.tstat.beta,'Coef'},{stats1.tstat.se,'StdErr'},{stats1.tstat.t,'tStat'},{stats1.tstat.pval,'pVal'});

stats2 = regstats(Price(plag+1:end),YQP);  
CoeffTable2 = dataset({stats2.tstat.beta,'Coef'},{stats2.tstat.se,'StdErr'},{stats2.tstat.t,'tStat'},{stats2.tstat.pval,'pVal'});
%--------------------------------------------------------------------------

% construct augmented VAR with block exogeneity  
%--------------------------------------------------------------------------
% step 1: construct unrestricted coefficient matrices
beta1 = stats1.beta'; beta2 = stats2.beta';
beta = [beta1(2:end); beta2(2:end)];

for i = 1:1:plag;
    AR{i} = [EstSpec.AR{i} zeros(n,2); beta(:,(i-1)*(n+2)+1:i*(n+2))];
end;
a = [EstSpec.a; beta(:,1)];

% step 2: obtain residual error covariance matrix 
resid = [W stats1.r stats2.r];
V = cov(resid,1);

% step 3: create augmented VAR model structure 
varmodel = vgxset('a',a,'AR',AR,'Q',V)
index(j)

% step 4: compute lower triangular factorization 
[A0,D] = triang_fact(V);

% step 5: obtain structural coefficient matrices 
for i = 1:1:plag;
   A{i} = A0\AR{i}; 
end;
c = A0\a; 

% step 5: pack results into an output structure
spec.A0 = inv(A0); spec.A  = A; spec.c  = c; spec.D  = D;
%--------------------------------------------------------------------------

% compute variance decompositions
%--------------------------------------------------------------------------
EstSpecVD.Q   = varmodel.Q;
EstSpecVD.AR  = varmodel.AR;
EstSpecVD.a   = varmodel.a;
EstSpecVD.nAR = varmodel.nAR;
EstSpecVD.n   = varmodel.n;

StructSpecVD  = recursiveness(EstSpecVD);

VD = zeros(2,16);
for tau = 1:1:16;
    Decomp    = vardecomp(EstSpecVD,StructSpecVD,6,tau);
    VD(:,tau) = Decomp.fevd([8 9]); 
end;
    
VD_II(index(j),:) = VD(1,:);
VD_QQ(index(j),:) = VD(2,:);
%--------------------------------------------------------------------------

end;

                                                                      

% variance decompositions for private fixed investment
%--------------------------------------------------------------------------
period = 1:1:16;

figure
subplot(4,2,1)
plot(period,VD_II(3,:),'k-',...    
       period,VD_II(5,:),'k:',period,VD_II(8,:),'k:',period,VD_II(9,:),'k:',...
       period,VD_II(10,:),'k:',period,VD_II(11,:),'k:',period,VD_II(12,:),'k:',...
       period,VD_II(13,:),'k:',period,VD_II(14,:),'k:',period,VD_II(15,:),'k:',...
       period,VD_II(18,:),'k:',period,VD_II(19,:),'k:',period,VD_II(20,:),'k:',...
       period,VD_II(22,:),'k:',period,VD_II(23,:),'k:',period,VD_II(25,:),'k:',...
       period,VD_II(26,:),'k:',period,VD_II(27,:),'k:',period,VD_II(28,:),'k:',...
       period,VD_II(30,:),'k:',period,VD_II(31,:),'k:',period,VD_II(32,:),'k:',...
       period,VD_II(33,:),'k:',period,VD_II(34,:),'k:'), 
title('\bf{Nonresidential Structures - Quantities}','FontSize',8)
xlabel('forecast horizon','FontSize',8)
ylabel('Percent','FontSize',8)
set(gca,'YAxisLocation','right','FontSize',8), xlim([0 16.5]), ylim([0 30]),
box off     
subplot(4,2,2)
plot(period,VD_QQ(3,:),'k-',...    
       period,VD_QQ(5,:),'k:',period,VD_QQ(8,:),'k:',period,VD_QQ(9,:),'k:',...
       period,VD_QQ(10,:),'k:',period,VD_QQ(11,:),'k:',period,VD_QQ(12,:),'k:',...
       period,VD_QQ(13,:),'k:',period,VD_QQ(14,:),'k:',period,VD_QQ(15,:),'k:',...
       period,VD_QQ(18,:),'k:',period,VD_QQ(19,:),'k:',period,VD_QQ(20,:),'k:',...
       period,VD_QQ(22,:),'k:',period,VD_QQ(23,:),'k:',period,VD_QQ(25,:),'k:',...
       period,VD_QQ(26,:),'k:',period,VD_QQ(27,:),'k:',period,VD_QQ(28,:),'k:',...
       period,VD_QQ(30,:),'k:',period,VD_QQ(31,:),'k:',period,VD_QQ(32,:),'k:',...
       period,VD_QQ(33,:),'k:',period,VD_QQ(34,:),'k:'), 
title('\bf{Nonresidential Structures - Prices}','FontSize',8)
xlabel('forecast horizon','FontSize',8)
ylabel('Percent','FontSize',8)
set(gca,'YAxisLocation','right','FontSize',8), xlim([0 16.5]), ylim([0 30]),
box off  

subplot(4,2,3)
plot(period,VD_II(85,:),'k-',...
    period,VD_II(87,:),'k:',period,VD_II(88,:),'k:',period,VD_II(90,:),'k:',...
    period,VD_II(91,:),'k:',period,VD_II(92,:),'k:',period,VD_II(93,:),'k:'), 
title('\bf{Residential Structures - Quantities}','FontSize',8)
xlabel('forecast horizon','FontSize',8)
ylabel('Percent','FontSize',8)
set(gca,'YAxisLocation','right','FontSize',8), xlim([0 16.5]), ylim([0 40]),
box off     
subplot(4,2,4)
plot(period,VD_QQ(85,:),'k-',...
    period,VD_QQ(87,:),'k:',period,VD_QQ(88,:),'k:',period,VD_QQ(90,:),'k:',...
    period,VD_QQ(91,:),'k:',period,VD_QQ(92,:),'k:',period,VD_QQ(93,:),'k:'), 
title('\bf{Residential Structures - Prices}','FontSize',8)
xlabel('forecast horizon','FontSize',8)
ylabel('Percent','FontSize',8)
set(gca,'YAxisLocation','right','FontSize',8), xlim([0 16.5]), ylim([0 40]),
box off  

subplot(4,2,5)
plot(period,VD_II(100,:),'k-',...    
       period,VD_II(38,:),'k:',period,VD_II(39,:),'k:',period,VD_II(41,:),'k:',...
       period,VD_II(42,:),'k:',period,VD_II(43,:),'k:',period,VD_II(44,:),'k:',...
       period,VD_II(45,:),'k:',period,VD_II(47,:),'k:',period,VD_II(49,:),'k:',...
       period,VD_II(50,:),'k:',period,VD_II(51,:),'k:',period,VD_II(52,:),'k:',...
       period,VD_II(53,:),'k:',period,VD_II(54,:),'k:',period,VD_II(56,:),'k:',...
       period,VD_II(59,:),'k:',period,VD_II(60,:),'k:',period,VD_II(61,:),'k:',...
       period,VD_II(62,:),'k:',period,VD_II(65,:),'k:',period,VD_II(66,:),'k:',...
       period,VD_II(68,:),'k:',period,VD_II(69,:),'k:',period,VD_II(71,:),'k:',...
       period,VD_II(72,:),'k:',period,VD_II(73,:),'k:',period,VD_II(74,:),'k:',...
       period,VD_II(76,:),'k:',period,VD_II(77,:),'k:',period,VD_II(78,:),'k:',...
       period,VD_II(79,:),'k:',period,VD_II(95,:),'k:'), 
title('\bf{Equipment - Quantities}','FontSize',8)
xlabel('forecast horizon','FontSize',8)
ylabel('Percent','FontSize',8)
set(gca,'YAxisLocation','right','FontSize',8), xlim([0 16.5]), ylim([0 30]),
box off     
subplot(4,2,6)
plot(period,VD_QQ(100,:),'k-',...    
       period,VD_QQ(38,:),'k:',period,VD_QQ(39,:),'k:',period,VD_QQ(41,:),'k:',...
       period,VD_QQ(42,:),'k:',period,VD_QQ(43,:),'k:',period,VD_QQ(44,:),'k:',...
       period,VD_QQ(45,:),'k:',period,VD_QQ(47,:),'k:',period,VD_QQ(49,:),'k:',...
       period,VD_QQ(50,:),'k:',period,VD_QQ(51,:),'k:',period,VD_QQ(52,:),'k:',...
       period,VD_QQ(53,:),'k:',period,VD_QQ(54,:),'k:',period,VD_QQ(56,:),'k:',...
       period,VD_QQ(59,:),'k:',period,VD_QQ(60,:),'k:',period,VD_QQ(61,:),'k:',...
       period,VD_QQ(62,:),'k:',period,VD_QQ(65,:),'k:',period,VD_QQ(66,:),'k:',...
       period,VD_QQ(68,:),'k:',period,VD_QQ(69,:),'k:',period,VD_QQ(71,:),'k:',...
       period,VD_QQ(72,:),'k:',period,VD_QQ(73,:),'k:',period,VD_QQ(74,:),'k:',...
       period,VD_QQ(76,:),'k:',period,VD_QQ(77,:),'k:',period,VD_QQ(78,:),'k:',...
       period,VD_QQ(79,:),'k:',period,VD_QQ(95,:),'k:'), 
title('\bf{Equipment - Prices}','FontSize',8)
xlabel('forecast horizon','FontSize',8)
ylabel('Percent','FontSize',8)
set(gca,'YAxisLocation','right','FontSize',8), xlim([0 16.5]), ylim([0 30]),
box off

subplot(4,2,7)
plot(period,VD_II(80,:),'k-',...
    period,VD_II(81,:),'k:',period,VD_II(82,:),'k:',period,VD_II(83,:),'k:'), 
title('\bf{Intellectual Property - Quantities}','FontSize',8)
xlabel('forecast horizon','FontSize',8)
ylabel('Percent','FontSize',8)
set(gca,'YAxisLocation','right','FontSize',8), xlim([0 16.5]), ylim([0 10]),
box off     
subplot(4,2,8)
plot(period,VD_QQ(80,:),'k-',...
    period,VD_QQ(81,:),'k:',period,VD_QQ(82,:),'k:',period,VD_QQ(83,:),'k:'), 
title('\bf{Intellectual Property - Prices}','FontSize',8)
xlabel('forecast horizon','FontSize',8)
ylabel('Percent','FontSize',8)
set(gca,'YAxisLocation','right','FontSize',8), xlim([0 16.5]), ylim([0 10]),
box off  
%--------------------------------------------------------------------------











