clc; format short g; clear all; close all;

% import data from Excel
X = importdata('dataset.xlsx')


% define variables for the VAR
%--------------------------------------------------------------------------
t0 = 1;                                                                     % start date: 1960:Q1 
T = 235;                                                                    % end data: (196=2007:Q4) (235=2017:Q3)

% general macro variables
y    = log(X.data.Macro(t0:T,1))*100;                                       % y    = log of real GDP
p    = log(X.data.Macro(t0:T,2)/100)*100;                                   % p    = log of chain type GDP Price Index

i    = (log(X.data.Real(1,t0:T))*100)';                                     % i = log of real private fixed investment
q    = (log(X.data.Price(1,t0:T)/100)*100)';                                % q = log of price index for private fixed investment

ff   = X.data.Macro(t0:T,3);                                                % ff   = shadow federal funds rate (Wu-Xia)    
m1   = log(X.data.Macro(t0:T,6))*100;                                       % m1   = log of M1 money stock

ppi_fuel = log(X.data.Macro(t0:T,4)/100)*100;                               % ppi_mat = log of PPI (fuels and related products) NSA
ppi_fin  = log(X.data.Macro(t0:T,5)/100)*100;                               % ppi_fin = log of PPI (final demand for finished goods) SA
%--------------------------------------------------------------------------


% create a VAR(p) specification and estimate
%--------------------------------------------------------------------------
Y = [y p i q ppi_fuel-ppi_fin ff m1];

% VAR(4) with constant intercept 
%-------------------------------
Spec = vgxset('n',size(Y,2),'nAR',4,'Constant',true)
n = Spec.n; plag = Spec.nAR;

[EstSpec,EstStdErrors,LLF,W] = vgxvarx(Spec,Y(plag+1:end,:),[],Y(1:plag,:));        % Warning! By default vgxvarx.m pads presample observations with zeros.      
EstSpec
%-------------------------------
%--------------------------------------------------------------------------


SD_irfII = zeros(500,21);
SD_irfQQ = zeros(500,21);

N = 64;

irf_II = zeros(N,21); irf_QQ = zeros(N,21);

index = [5 8 9 10 11 12 13 14 15 18 19 20 22 23 25 26 27 28 30 31 32 33 34 ...                                 %1-23  (nonresidential structures)
           38 39 41 42 43 44 45 47 49 50 51 52 53 54 56 59 60 61 62 65 66 68 69 71 72 73 74 76 77 78 79 ...    %24-54 (nonresidential equipment)
           81 82 83 ...                                                                                        %55-57 (intellectual property products)
           87 88 90 91 92 93 ...                                                                               %58-63 (residential structures)
           95];                                                                                                %64    (residential equipment)
       
  
for u = 1:1:500;       
       
for j = 1:1:N;

% estimate industry-level regressions
%--------------------------------------------------------------------------
Quantity = (log(X.data.Real(index(j),t0:T))*100)';
Price    = (log(X.data.Price(index(j),t0:T))*100)';

Q1 = lag(Quantity,1); Q2 = lag(Quantity,2); Q3 = lag(Quantity,3); Q4 = lag(Quantity,4); 
P1 = lag(Price,1);    P2 = lag(Price,2);    P3 = lag(Price,3);    P4 = lag(Price,4);
Y1 = lag(Y,1);        Y2 = lag(Y,2);        Y3 = lag(Y,3);        Y4 = lag(Y,4);

YQP = [Y1 Q1 P1 Y2 Q2 P2 Y3 Q3 P3 Y4 Q4 P4]; 
YQP = YQP(plag+1:end,:);

stats1 = regstats(Quantity(plag+1:end),YQP);  
CoeffTable1 = dataset({stats1.tstat.beta,'Coef'},{stats1.tstat.se,'StdErr'},{stats1.tstat.t,'tStat'},{stats1.tstat.pval,'pVal'});

stats2 = regstats(Price(plag+1:end),YQP);  
CoeffTable2 = dataset({stats2.tstat.beta,'Coef'},{stats2.tstat.se,'StdErr'},{stats2.tstat.t,'tStat'},{stats2.tstat.pval,'pVal'});
%--------------------------------------------------------------------------       
       
% bootstrap the industry residuals and simulate
%--------------------------------------------------------------------------
nresid    = length(stats1.r);
rng('shuffle')
bootindex = randi(nresid,nresid,1);

resid1 = stats1.r(bootindex);
resid2 = stats2.r(bootindex);

simul1 = [i(1:plag); [i(plag+1:end) + resid1]];
simul2 = [(q(1:plag) + 100*log(100)); [(q(plag+1:end) + 100*log(100)) + resid2]];
%--------------------------------------------------------------------------     

% re-estimate industry-level regressions with simulated data
%--------------------------------------------------------------------------
Quantity = simul1;
Price    = simul2;

Q1 = lag(Quantity,1); Q2 = lag(Quantity,2); Q3 = lag(Quantity,3); Q4 = lag(Quantity,4); 
P1 = lag(Price,1);    P2 = lag(Price,2);    P3 = lag(Price,3);    P4 = lag(Price,4);

YQP = [Y1 Q1 P1 Y2 Q2 P2 Y3 Q3 P3 Y4 Q4 P4]; 
YQP = YQP(plag+1:end,:);

stats1 = regstats(Quantity(plag+1:end),YQP);  
CoeffTable1 = dataset({stats1.tstat.beta,'Coef'},{stats1.tstat.se,'StdErr'},{stats1.tstat.t,'tStat'},{stats1.tstat.pval,'pVal'});

stats2 = regstats(Price(plag+1:end),YQP);  
CoeffTable2 = dataset({stats2.tstat.beta,'Coef'},{stats2.tstat.se,'StdErr'},{stats2.tstat.t,'tStat'},{stats2.tstat.pval,'pVal'});
%--------------------------------------------------------------------------

% construct augmented VAR with block exogeneity  
%--------------------------------------------------------------------------
% step 1: construct unrestricted coefficient matrices
beta1 = stats1.beta'; beta2 = stats2.beta';
beta = [beta1(2:end); beta2(2:end)];

for z = 1:1:plag;
    AR{z} = [EstSpec.AR{z} zeros(n,2); beta(:,(z-1)*(n+2)+1:z*(n+2))];
end;
a = [EstSpec.a; beta(:,1)];

% step 2: obtain residual error covariance matrix 
resid = [W stats1.r stats2.r];
V = cov(resid,1);

% step 3: create augmented VAR model structure 
varmodel = vgxset('a',a,'AR',AR,'Q',V);

% step 4: compute lower triangular factorization 
[A0,D] = triang_fact(V);

% step 5: obtain structural coefficient matrices 
for z = 1:1:plag;
   A{z} = A0\AR{z}; 
end;
c = A0\a; 

% step 5: pack results into an output structure
spec.A0 = inv(A0); spec.A  = A; spec.c  = c; spec.D  = D;
%--------------------------------------------------------------------------

% apply recursive identification of monetary policy shocks
%--------------------------------------------------------------------------
DynRespFunc = impulse(varmodel,spec,6,20);

irf_ii = -DynRespFunc.Phi{8};  
irf_qq = -DynRespFunc.Phi{9};  
%--------------------------------------------------------------------------
       
irf_II(j,:) = irf_ii;
irf_QQ(j,:) = irf_qq;

end;

SD_irfII(u,:) = std(irf_II);
SD_irfQQ(u,:) = std(irf_QQ);

end;

II = mean(SD_irfII); save II II
QQ = mean(SD_irfQQ); save QQ QQ
       
       
       
       
       
       
       


