clc; format short g; clear all; close all;

% import data from Excel
X = importdata('dataset.xlsx')


% define variables for the VAR
%--------------------------------------------------------------------------
t0 = 1;                                                                     % start date: 1960:Q1 
T = 235;                                                                    % end data: (196=2007:Q4) (235=2017:Q3)

% general macro variables
y    = log(X.data.Macro(t0:T,1))*100;                                       % y    = log of real GDP
p    = log(X.data.Macro(t0:T,2)/100)*100;                                   % p    = log of chain type GDP Price Index

i    = (log(X.data.Real(1,t0:T))*100)';                                     % i = log of real private fixed investment
q    = (log(X.data.Price(1,t0:T)/100)*100)';                                % q = log of price index for private fixed investment

ff   = X.data.Macro(t0:T,3);                                                % ff   = shadow federal funds rate (Wu-Xia)    
m1   = log(X.data.Macro(t0:T,6))*100;                                       % m1   = log of M1 money stock

ppi_fuel = log(X.data.Macro(t0:T,4)/100)*100;                               % ppi_mat = log of PPI (fuels and related products) NSA
ppi_fin  = log(X.data.Macro(t0:T,5)/100)*100;                               % ppi_fin = log of PPI (final demand for finished goods) SA
%--------------------------------------------------------------------------




index = [5 8 9 10 11 12 13 14 15 18 19 20 22 23 25 26 27 28 30 31 32 33 34 ...                                 %1-23  (nonresidential structures)
           38 39 41 42 43 44 45 47 49 50 51 52 53 54 56 59 60 61 62 65 66 68 69 71 72 73 74 76 77 78 79 ...    %24-54 (nonresidential equipment)
           81 82 83 ...                                                                                        %55-57 (intellectual property products)
           87 88 90 91 92 93 ...                                                                               %58-63 (residential structures)
           95]; 
       
N      = length(index);
IRF_II = zeros(N,21);
IRF_QQ = IRF_II;

for j = 1:1:N;

%--------------------------------------------------------------------------
Quantity = (log(X.data.Real(index(j),t0:T))*100)';
Price    = (log(X.data.Price(index(j),t0:T))*100)';

% create a VAR(p) specification and estimate
%--------------------------------------------------------------------------
Y = [y p i q ppi_fuel-ppi_fin ff m1 Quantity Price];

% VAR(4) with constant intercept 
%-------------------------------
Spec = vgxset('n',size(Y,2),'nAR',4,'Constant',true)
n = Spec.n; plag = Spec.nAR;

[EstSpec,EstStdErrors,LLF,W] = vgxvarx(Spec,Y(plag+1:end,:),[],Y(1:plag,:));        % Warning! By default vgxvarx.m pads presample observations with zeros.      
EstSpec
index(j)
%-------------------------------
%--------------------------------------------------------------------------

% apply recursive identification of monetary policy shocks
%--------------------------------------------------------------------------
StructSpec = recursiveness(EstSpec);

DynRespFunc = impulse(EstSpec,StructSpec,6,20);
%--------------------------------------------------------------------------

% record confidence bands for impulse response functions
%--------------------------------------------------------------------------
irf_ii = -DynRespFunc.Phi{8}; 
irf_qq = -DynRespFunc.Phi{9}; 

IRF_II(j,:) = irf_ii;
IRF_QQ(j,:) = irf_qq;
%--------------------------------------------------------------------------

end;


IImean_2 = mean(IRF_II); save IImean_2 IImean_2
QQmean_2 = mean(IRF_QQ); save QQmean_2 QQmean_2
IIstdv_2 = std(IRF_II);  save IIstdv_2 IIstdv_2
QQstdv_2 = std(IRF_QQ);  save QQstdv_2 QQstdv_2




% Note: unstable VAR - 23. Mining structures 32. Farm structures 
%                      33. Other structures 95. Residential equipment











