clc; format short g; clear all; close all;

% import data from Excel
X = importdata('dataset.xlsx')


% define variables for the VAR
%--------------------------------------------------------------------------
t0 = 85;                                                                    % start date: 1981:Q1 
T = 196;                                                                    % end data: (196=2007:Q4) (235=2017:Q3)

% general macro variables
y    = log(X.data.Macro(t0:T,1))*100;                                       % y    = log of real GDP
p    = log(X.data.Macro(t0:T,2)/100)*100;                                   % p    = log of chain type GDP Price Index

i    = (log(X.data.Real(1,t0:T))*100)';                                     % i = log of real private fixed investment
q    = (log(X.data.Price(1,t0:T)/100)*100)';                                % q = log of price index for private fixed investment

ff   = X.data.Macro(t0:T,3);                                                % ff   = shadow federal funds rate (Wu-Xia)    
m1   = log(X.data.Macro(t0:T,6))*100;                                       % m1   = log of M1 money stock

ppi_fuel = log(X.data.Macro(t0:T,4)/100)*100;                               % ppi_mat = log of PPI (fuels and related products) NSA
ppi_fin  = log(X.data.Macro(t0:T,5)/100)*100;                               % ppi_fin = log of PPI (final demand for finished goods) SA
%--------------------------------------------------------------------------


% create a VAR(p) specification and estimate
%--------------------------------------------------------------------------
Y = [y p i q ppi_fuel-ppi_fin ff m1];

% VAR(4) with constant intercept 
%-------------------------------
Spec = vgxset('n',size(Y,2),'nAR',4,'Constant',true)
n = Spec.n; plag = Spec.nAR;

[EstSpec,EstStdErrors,LLF,W] = vgxvarx(Spec,Y(plag+1:end,:),[],Y(1:plag,:));        % Warning! By default vgxvarx.m pads presample observations with zeros.      
EstSpec
%-------------------------------
%--------------------------------------------------------------------------




index = [5 8 9 10 11 12 13 14 15 18 19 20 22 23 25 26 27 28 30 31 32 33 34 ...                                 %1-23  (nonresidential structures)
           38 39 41 42 43 44 45 47 49 50 51 52 53 54 56 59 60 61 62 65 66 68 69 71 72 73 74 76 77 78 79 ...    %24-54 (nonresidential equipment)
           81 82 83 ...                                                                                        %55-57 (intellectual property products)
           87 88 90 91 92 93 ...                                                                               %58-63 (residential structures)
           95]; 
       
N      = length(index);
IRF_II = zeros(N,21);
IRF_QQ = IRF_II;

for j = 1:1:N;
    
% estimate industry-level regressions
%--------------------------------------------------------------------------    
Quantity = (log(X.data.Real(index(j),t0:T))*100)';
Price    = (log(X.data.Price(index(j),t0:T))*100)';

Q1 = lag(Quantity,1); Q2 = lag(Quantity,2); Q3 = lag(Quantity,3); Q4 = lag(Quantity,4); 
P1 = lag(Price,1);    P2 = lag(Price,2);    P3 = lag(Price,3);    P4 = lag(Price,4);
Y1 = lag(Y,1);        Y2 = lag(Y,2);        Y3 = lag(Y,3);        Y4 = lag(Y,4);

YQP = [Y1 Q1 P1 Y2 Q2 P2 Y3 Q3 P3 Y4 Q4 P4]; 
YQP = YQP(plag+1:end,:);

stats1 = regstats(Quantity(plag+1:end),YQP);  
CoeffTable1 = dataset({stats1.tstat.beta,'Coef'},{stats1.tstat.se,'StdErr'},{stats1.tstat.t,'tStat'},{stats1.tstat.pval,'pVal'});

stats2 = regstats(Price(plag+1:end),YQP);  
CoeffTable2 = dataset({stats2.tstat.beta,'Coef'},{stats2.tstat.se,'StdErr'},{stats2.tstat.t,'tStat'},{stats2.tstat.pval,'pVal'});
%--------------------------------------------------------------------------

% construct augmented VAR with block exogeneity  
%--------------------------------------------------------------------------
% step 1: construct unrestricted coefficient matrices
beta1 = stats1.beta'; beta2 = stats2.beta';
beta = [beta1(2:end); beta2(2:end)];

for i = 1:1:plag;
    AR{i} = [EstSpec.AR{i} zeros(n,2); beta(:,(i-1)*(n+2)+1:i*(n+2))];
end;
a = [EstSpec.a; beta(:,1)];

% step 2: obtain residual error covariance matrix 
resid = [W stats1.r stats2.r];
V = cov(resid,1);

% step 3: create augmented VAR model structure 
varmodel = vgxset('a',a,'AR',AR,'Q',V)
index(j)

% step 4: compute lower triangular factorization 
[A0,D] = triang_fact(V);

% step 5: obtain structural coefficient matrices 
for i = 1:1:plag;
   A{i} = A0\AR{i}; 
end;
c = A0\a; 

% step 5: pack results into an output structure
spec.A0 = inv(A0); spec.A  = A; spec.c  = c; spec.D  = D;
%--------------------------------------------------------------------------

% apply recursive identification of monetary policy shocks
%--------------------------------------------------------------------------
DynRespFunc = impulse(varmodel,spec,6,20);
%--------------------------------------------------------------------------

% record confidence bands for impulse response functions
%--------------------------------------------------------------------------
irf_ii = -DynRespFunc.Phi{8}; 
irf_qq = -DynRespFunc.Phi{9}; 

IRF_II(j,:) = irf_ii;
IRF_QQ(j,:) = irf_qq;
%--------------------------------------------------------------------------

end;


IImean_1 = mean(IRF_II); save IImean_1 IImean_1
QQmean_1 = mean(IRF_QQ); save QQmean_1 QQmean_1
IIstdv_1 = std(IRF_II);  save IIstdv_1 IIstdv_1
QQstdv_1 = std(IRF_QQ);  save QQstdv_1 QQstdv_1




% Note: unstable VAR - 39. Communication equipment 56. Trucks, buses, and truck trailers
















