function StructSpec = recursiveness(EstSpec);
%--------------------------------------------------------------------------
% PURPOSE: performs recursive (lower triangular) identification of a VAR         
%--------------------------------------------------------------------------
% USAGE:   StructSpec = recursiveness(EstSpec) 
% where:   EstSpec    = a vgxvarx.m output 'structure'
%--------------------------------------------------------------------------
% RETURNS a 'structure'
% StructSpec.A0    = LHS lower triangular matrix of a VAR of form:
%                    A0*y(t) = c + A1*y(t-1) + ... + Aq*y(t-q) + e(t)
% StructSpec.A     = RHS matrices A{i} = Ai for i = 1,...,q 
% StructSpec.c     = RHS intercept vector c 
% StructSpec.D     = structural error coveriance matrix Cov(e(t))
%--------------------------------------------------------------------------
% written by: Gregory Givens
%             27 February 2013
%--------------------------------------------------------------------------


StructSpec.method = 'recursive (lower triangular) identification';

%--------------------------------------------------------------------------
% Step 1: obtain residual error covariance matrix 
V = EstSpec.Q;

% Step 2: compute lower triangular factorization of V
[A0,D] = triang_fact(V);

% Step 3: obtain structural error covariance matrix  
% D = D;

% Step 4: obtain structural coefficient matrices
for i = 1:1:EstSpec.nAR;
   A{i} = A0\EstSpec.AR{i};
end;

% Step 5: obtain structural intercept terms (if any)
logic = isempty(EstSpec.a);
if logic == 1;
    c = [];
end;
if logic == 0;
    c = A0\EstSpec.a;
end;
%--------------------------------------------------------------------------

% Pack results into an output structure
StructSpec.A0 = inv(A0);
StructSpec.A  = A;
StructSpec.c  = c;
StructSpec.D  = D;