function [S,dd] = sylvester(Sigma,M,beta)
%--------------------------------------------------------------------------
% PURPOSE: solves the discrete Lyapunov or Sylvester equation          
%--------------------------------------------------------------------------
% USAGE:  S      = sylvester(Sigma,M,beta)
% where:  Sigma  = nxn matrix 
%         M      = nxn matrix 
%         beta   = real-valued scalar (often equal to one)
% RETURNS: a nxn matrix S
%--------------------------------------------------------------------------  
%  Sylvester equation:
%  S = Sigma + beta*M*S*M'
%--------------------------------------------------------------------------
% written by: Gregory Givens
%             25 May 2006
%--------------------------------------------------------------------------


[m n] = size(Sigma);
[q r] = size(M);

% check matrix dimensions
%--------------------------------------------------------------------------
if m ~= n;
    error('Sigma matrix must be square');
end;
if q ~= r;
    error('M matrix must be square');
end;
if m ~= q & n ~= r;
    error('Sigma and M matrices must be of same dimension');
end;
%--------------------------------------------------------------------------

% solve the fixed point problem for S
%--------------------------------------------------------------------------
St = eye(m)*.1;
dd = 1; tol = 10^(-8); count = 0; maxit = 10000;
while (dd>tol)&(count<maxit);
   Sto = St;
   St = Sigma + beta*M*St*M';
   count = count+1;
   dd = norm(max(abs(Sto-St))); 
end;
if count >= maxit;
   disp('max number of sylvester iterations reached');
end;
S = St;
%--------------------------------------------------------------------------



